@@ -11,6 +11,7 @@ import os
 import json
 import glob
 import rasterio
+from rasterio.crs import CRS
 import matplotlib.pyplot as plt
 from copy import copy
 from os.path import join, normpath, isfile
@@ -18,17 +19,24 @@ from descartes import PolygonPatch
 from rasterio.plot import show
 from mpl_toolkits.axes_grid1 import ImageGrid
 
-from cbm.utils import config, spatial_utils
-from cbm.get import background as get_bg
-from cbm.get import parcel_info
+from cbm.utils import config
+from cbm.get import parcel_info, background
 
 
-def overlay_parcel(img, geom):
+def overlay_parcel(img, parcel, debug=False):
     """Create parcel polygon overlay"""
-    patche = [PolygonPatch(feature, edgecolor="yellow",
-                           facecolor="none", linewidth=2
-                           ) for feature in geom['geom']]
-    return patche
+    img_epsg = img.crs.to_epsg()
+    if debug:
+        print('img_epsg: ', img_epsg)
+    parcel['geom'] = [rasterio.warp.transform_geom(
+        CRS.from_epsg(parcel['srid'][0]),
+        CRS.from_epsg(img_epsg),
+        feature,  precision=6
+    ) for feature in parcel['geom']]
+    patches = [PolygonPatch(feature, edgecolor="yellow",
+                            facecolor="none", linewidth=2
+                            ) for feature in parcel['geom']]
+    return patches
 
 
 def by_location(aoi, year, lon, lat, chipsize=512, extend=512, tms=['google'],
@@ -63,6 +71,8 @@ def by_location(aoi, year, lon, lat, chipsize=512, extend=512, tms=['google'],
             pid = parcel['pid'][0]
         else:
             pid = parcel['pid']
+        if type(parcel['geom'][0]) is str:
+            parcel['geom'] = [json.loads(g) for g in parcel['geom']]
 
         workdir = normpath(join(config.get_value(['paths', 'temp']),
                                 aoi, str(year), str(pid)))
@@ -85,23 +95,22 @@ def by_location(aoi, year, lon, lat, chipsize=512, extend=512, tms=['google'],
               extend, tms, ptype, columns, debug)
 
     for t in tms:
-        if not isfile(normpath(join(bg_path, f'{t.lower()}.tif'))) or not same_args:
-            if parcel_id:
-                get_bg.by_pid(aoi, year, pid, chipsize,
-                              extend, t, ptype, True, debug)
-            else:
-                get_bg.by_location(aoi, year, lon, lat, chipsize,
-                                   extend, t, ptype, True, debug)
+        try:
+            if not isfile(normpath(join(bg_path, f'{t.lower()}.tif'))) or not same_args:
+                if parcel_id:
+                    background.by_pid(aoi, year, pid, chipsize,
+                                      extend, t, ptype, True, debug)
+                else:
+                    background.by_location(aoi, year, lon, lat, chipsize,
+                                           extend, t, ptype, True, debug)
+        except Exception as err:
+            print("Could not get image from '{t}', ", err)
 
     if parcel_id:
         with open(normpath(join(workdir, 'info.json')), 'r') as f:
             parcel = json.load(f)
-
-        with rasterio.open(normpath(join(bg_path,
-                                         f'{tms[0].lower()}.tif'))) as img:
-            img_epsg = img.crs.to_epsg()
-            geom = spatial_utils.transform_geometry(parcel, img_epsg)
-            patches = overlay_parcel(img, geom)
+            if type(parcel['geom'][0]) is str:
+                parcel['geom'] = [json.loads(g) for g in parcel['geom']]
 
     rows = int(len(tms) // columns + (len(tms) % columns > 0))
     fig = plt.figure(figsize=(30, 10 * rows))
@@ -118,9 +127,13 @@ def by_location(aoi, year, lon, lat, chipsize=512, extend=512, tms=['google'],
             bbox=dict(boxstyle="round", ec='yellow', fc='black', alpha=0.2))
         return date_text
 
+    with rasterio.open(normpath(join(bg_path, f'{tms[0].lower()}.tif'))) as img:
+        patches = overlay_parcel(img, parcel, debug)
+
     for ax, t in zip(grid, tms):
         with rasterio.open(normpath(join(bg_path, f'{t.lower()}.tif'))) as img:
             if parcel_id:
+                # patches = overlay_parcel(img, parcel, debug)
                 for patch in patches:
                     ax.add_patch(copy(patch))
             overlay_title(img, t)
@@ -174,17 +187,17 @@ def by_pid(aoi, year, pid, chipsize=512, extend=512, tms=['google'],
         print(aoi, year, pid, chipsize, extend, tms, columns, debug)
 
     for t in tms:
-        if not isfile(normpath(join(bg_path, f'{t.lower()}.tif'))) or not same_args:
-            get_bg.by_pid(aoi, year, pid, chipsize,
-                          extend, t, ptype, True, debug)
+        try:
+            if not isfile(normpath(join(bg_path, f'{t.lower()}.tif'))) or not same_args:
+                background.by_pid(aoi, year, pid, chipsize,
+                                  extend, t, ptype, True, debug)
+        except Exception as err:
+            print(f"Could not get image from '{t}', ", err)
 
     with open(file_info, 'r') as f:
         parcel = json.load(f)
-
-    with rasterio.open(normpath(join(bg_path, f'{tms[0].lower()}.tif'))) as img:
-        img_epsg = img.crs.to_epsg()
-        geom = spatial_utils.transform_geometry(parcel, img_epsg)
-        patches = overlay_parcel(img, geom)
+        if type(parcel['geom'][0]) is str:
+            parcel['geom'] = [json.loads(g) for g in parcel['geom']]
 
     rows = int(len(tms) // columns + (len(tms) % columns > 0))
     fig = plt.figure(figsize=(30, 10 * rows))
@@ -200,13 +213,20 @@ def by_pid(aoi, year, pid, chipsize=512, extend=512, tms=['google'],
             bbox=dict(boxstyle="round", ec='yellow', fc='black', alpha=0.2))
         return date_text
 
+    with rasterio.open(normpath(join(bg_path, f'{tms[0].lower()}.tif'))) as img:
+        patches = overlay_parcel(img, parcel, debug)
+
     for ax, t in zip(grid, tms):
-        with rasterio.open(normpath(join(bg_path, f'{t.lower()}.tif'))) as img:
-            for patch in patches:
-                ax.add_patch(copy(patch))
-            overlay_title(img, t)
-            show(img, ax=ax)
-#            ax.xaxis.set_major_locator(ticker.MultipleLocator(200))
+        try:
+            with rasterio.open(normpath(join(bg_path, f'{t.lower()}.tif'))) as img:
+                # patches = overlay_parcel(img, parcel, debug)
+                for patch in patches:
+                    ax.add_patch(copy(patch))
+                overlay_title(img, t)
+                show(img, ax=ax)
+    #            ax.xaxis.set_major_locator(ticker.MultipleLocator(200))
+        except Exception as err:
+            print("Could not add overlay", err)
 
     if len(tms) > columns and columns * rows > len(tms):
         for ax in grid[-((columns * rows - len(tms))):]:
